import 'package:chat_widget/src/core/models/channel_model.dart';
import 'package:chat_widget/src/core/models/meetings_model.dart';
import 'package:chat_widget/src/core/models/message_model.dart';
import 'package:chat_widget/src/core/models/message_paginate.dart';
import 'package:chat_widget/src/core/services/chat_services.dart';
import 'package:flutter/material.dart';
import 'package:get/get_rx/get_rx.dart';
import 'package:get/get_state_manager/get_state_manager.dart';
import 'package:socket_io_client/socket_io_client.dart';

class ChatGetPackageController extends GetxController {
  late ChatServices chatServices;
  TextEditingController textEditingController = TextEditingController();
  ScrollController scrollController = ScrollController();

  String message = "";
  late int userId;
  late String companyUuid;
  late String url;
  bool isUser = true;
  int _count = 0;
  Socket? socket;
  bool active = true;

  RxBool loading = true.obs;
  RxBool loadingPaginate = false.obs;
  bool hasNext = false;
  int page = 1;

  RxList<MessageModel> listMessages = <MessageModel>[].obs;
  ChannelModel? channelModel;
  MeetingsModel? meetingsModel;

  ChatGetPackageController(
      {required this.userId,
      required this.companyUuid,
      required this.url,
      this.isUser = true});

  @override
  void onInit() {
    super.onInit();

    chatServices = ChatServices(url: url);

    getChannel();

    scrollController.addListener(() {
      final max = scrollController.position.maxScrollExtent;
      final offset = scrollController.offset;

      if ((max * 0.7) < offset && hasNext && !loadingPaginate.value) {
        getMessages();
        loadingPaginate.value = true;
      }
    });
  }

  void getChannel() async {
    channelModel = await chatServices.getChannel(userId, companyUuid);
    initizalizeSocket();
    await getMeeting();
    await getMessages();
    loading.value = false;
  }

  void initizalizeSocket() {
    try {
      if (channelModel != null) {
        socket = io(
          url,
          OptionBuilder()
              .setTransports(['websocket']) // for Flutter or Dart VM
              .enableAutoConnect()
              .build(),
        );
        if (socket != null) {
          print("socket ${socket!.connected}");
          socket!.connect();
          socket!.on("connect", (data) {
            print("conectado");
            _pingPong();

            socket!.emit("join", channelModel!.id);
            socket?.on("pong", (data) => _count = 0);

            socket!.on("new-meeting", (data) {
              meetingsModel = MeetingsModel.fromJson(data);
            });

            socket!.on("send_message", (data) {
              MessageModel message = MessageModel.fromJson(data);
              if (message.messageUserType !=
                  (isUser ? "user" : "colaborator")) {
                listMessages.insert(0, message);
              }
            });
          });
        }
      }
    } catch (e) {
      print("Error");
    }
  }

  getMeeting() async {
    if (channelModel != null) {
      meetingsModel = await chatServices.getMeeting(channelModel!.id);

      if (socket != null && meetingsModel != null) {
        socket!.emit("join", "meet-${meetingsModel!.id}");

        socket!.on("close_meet", (data) {
          meetingsModel = null;
        });
      }
    }
  }

  getMessages() async {
    if (channelModel != null) {
      MessagePaginate? response =
          await chatServices.getMessages(channelModel!.id, 15, page);

      if (response != null) {
        listMessages.addAll(response.data);
        hasNext = response.hasNext;
        page++;
        loadingPaginate.value = false;
      }
    }
  }

  void sendMessage() async {
    if (textEditingController.text != "") {
      final id = DateTime.now().millisecondsSinceEpoch;

      MessageModel messageModel = MessageModel(
          id: id,
          content: textEditingController.text,
          type: 1,
          userId: userId,
          channelId: channelModel!.id,
          messageUserType: isUser ? "user" : "colaborator",
          companyId: channelModel!.companyId,
          messageStatus: "wait",
          meetingId: meetingsModel?.id);

      textEditingController.text = "";
      listMessages.insert(0, messageModel);

      meetingsModel ??= await chatServices.createMeeting(
          channelModel!.id, channelModel!.companyId);

      messageModel.meetingId = meetingsModel!.id;

      final message = await chatServices.sendMessage(messageModel);

      if (message != null) {
        int index = listMessages.indexWhere((element) => element.id == id);

        if (index >= 0) {
          listMessages[index] = message;
        } else {
          listMessages.insert(0, message);
        }
      }
    }
  }

  void leaveChannel() {
    if (socket != null) {
      socket!.emit('leave_channel', channelModel!.id);
    }
  }

  void _pingPong() {
    if (active) {
      socket!.emit("pingpong");
      if (socket!.connected && _count < 10) {
        Future.delayed(const Duration(seconds: 1), () {
          _count++;
          _pingPong();
        });
      } else {
        _count = 0;
        socket!.close();
        socket = socket!.connect();

        Future.delayed(const Duration(seconds: 1), () {
          if (socket!.disconnected) {
            _count++;
            _pingPong();
          }
        });
      }
    }
  }

  @override
  void onClose() {
    super.onClose();
    active = false;
    socket!.disconnect();
    socket!.close();
  }
}
