import 'package:chat_widget/src/core/models/channel_model.dart';
import 'package:chat_widget/src/core/models/meetings_model.dart';
import 'package:chat_widget/src/core/models/message_model.dart';
import 'package:chat_widget/src/core/models/message_paginate.dart';
import 'package:dio/dio.dart';

class ChatServices {
  late String url;

  ChatServices({required this.url});

  Future<MessageModel?> sendMessage(MessageModel message) async {
    try {
      Response response = await Dio().post("$url/messages", data: message);

      return MessageModel.fromJson(response.data["data"]);
    } catch (e) {
      print(e);
      return null;
    }
  }

  Future<ChannelModel?> getChannel(userId, companyId) async {
    try {
      Response response = await Dio().post("$url/channels",
          data: {"user_id": userId, "company_uuid": companyId});

      return ChannelModel.fromJson(response.data["data"]);
    } catch (e) {
      print(e);
      return null;
    }
  }

  Future<MessagePaginate?> getMessages(companyId, limit, page) async {
    try {
      Response response =
          await Dio().get("$url/messages/$companyId?limit=$limit&page=$page");

      return MessagePaginate.fromJson(response.data);
    } catch (e) {
      print(e);
      return null;
    }
  }

  Future<MeetingsModel?> getMeeting(channelId) async {
    try {
      Response response = await Dio().get("$url/meeting/$channelId");

      return MeetingsModel.fromJson(response.data["data"]);
    } catch (e) {
      return null;
    }
  }

  Future<MeetingsModel?> createMeeting(channelId, companyId) async {
    try {
      Response response = await Dio().post("$url/meeting",
          data: {"company_id": companyId, "channel_id": channelId});

      return MeetingsModel.fromJson(response.data["data"]);
    } catch (e) {
      print(e);
      return null;
    }
  }
}
