import 'package:chat_widget/src/core/models/message_model.dart';
import 'package:chat_widget/src/global/app_colors.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:intl/intl.dart';

class MessageWidget extends StatelessWidget {
  const MessageWidget({
    required this.message,
    required this.isUser,
    Key? key,
  }) : super(key: key);

  final MessageModel message;
  final bool isUser;

  @override
  Widget build(BuildContext context) {
    final userType = isUser ? "user" : "colaborator";
    return Row(
      mainAxisAlignment: message.messageUserType == userType
          ? MainAxisAlignment.end
          : MainAxisAlignment.start,
      children: [
        Container(
          margin: const EdgeInsets.symmetric(vertical: 10).r,
          decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(20).r,
              color: message.messageUserType == userType
                  ? AppColors().green
                  : AppColors().second),
          padding: const EdgeInsets.symmetric(vertical: 20, horizontal: 20).r,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              Container(
                constraints: BoxConstraints(maxWidth: 500.r, minWidth: 200.r),
                child: Text(
                  message.content,
                  textAlign: TextAlign.left,
                  style: const TextStyle(
                      color: Colors.white,
                      fontSize: 18,
                      fontWeight: FontWeight.w600),
                ),
              ),
              Row(
                children: [
                  if (message.createdAt != null)
                    Text(
                      DateFormat.Hm().format(message.createdAt!),
                      style: const TextStyle(
                          color: Colors.white,
                          fontSize: 14,
                          fontWeight: FontWeight.w400),
                    ),
                  if (message.messageStatus == "wait")
                    Icon(
                      Icons.watch_later_outlined,
                      size: 45.r,
                      color: Colors.white,
                    ),
                  if (message.messageStatus == "save")
                    Icon(
                      Icons.check_sharp,
                      size: 45.r,
                      color: Colors.white,
                    ),
                ],
              )
            ],
          ),
        ),
      ],
    );
  }
}
